@@ -4,7 +4,6 @@ Name Server persistent storage implementations.
 Pyro - Python Remote Objects.  Copyright by Irmen de Jong (irmen@razorvine.net).
 """
 
-from __future__ import with_statement
 import re
 import logging
 import sys
@@ -155,8 +154,9 @@ class SqlStorage(MutableMapping):
                 db.execute("PRAGMA foreign_keys=ON")
                 db.execute("DELETE FROM pyro_metadata")
                 db.execute("DELETE FROM pyro_names")
-                db.execute("VACUUM")
                 db.commit()
+            with closing(sqlite3.connect(self.dbfile, isolation_level=None)) as db:
+                db.execute("VACUUM")  # this cannot run inside a transaction.
         except sqlite3.DatabaseError as e:
             raise NamingError("sqlite error in clear: "+str(e))
 
